unit uLanguage;

interface

uses XMLIntf, XMLDoc, Forms, Classes, Controls;

type
  TLanguageItem=class(TObject)
  private
    XML:IXMLDocument;
    function GetLngFileName: String;
    function GetLngName: String;
  public
    constructor Create(FileName:String);
    property LngName:String read GetLngName;
    property LngFileName:String read GetLngFileName;
  end;

type
  TLanguage=class(TObject)
  private
    XML:IXMLDocument;
    FLanguages:TStringList;
    function GetFileName: String;
    procedure LoadForms(NodE:IXMLNode);
    procedure BrowseComponent(Component:TControl; Node:IXMLNode);
    function getLanguages: TStringList;
    procedure LoadStringList(Component:TComponent;Node:IXMLNode);
    procedure TranslateFormOrFrame(FormOrFrame:TScrollingWinControl);
  public
    constructor create;
    destructor Destroy; override;
    Function text(key, defaultValue:string):String;
    procedure LoadLngFile(FileName: String);
    property FileName:String read GetFileName;
    procedure RewriteForm(Form:TForm);
    property Languages:TStringList read getLanguages;
    procedure RefreshLanguages;
    procedure TranslateFrame(Frame:TFrame);
    procedure TranslateForm(Form:TForm);
  end;

Procedure GenerateXmlLngFile(filename:string='');
Procedure SaveComponentBranch(Component:TControl; Node:IXMLNode);

var Language:TLanguage;

implementation

uses SysUtils, TypInfo, Dialogs, StdCtrls, uxmlutils;

const
  cDefaultXML_LNGFileName  = 'default.lng.xml';

  cDefaultXMLHeadEncoding = '<?xml version="1.0" encoding="windows-1250" ?>';
  cDefaultXMLRootNode     = '<language name="language"/>';
  cDefaultXMLhead         = cDefaultXMLHeadEncoding + cDefaultXMLRootNode;

  cLNGFileNotFount        = 'Soubor s pekladem:"%s" nenalezen!';

  cFormsElement      = 'forms';
  cConstElement      = 'const';
  cItemsElement      = 'items';
  cItemElement       = 'item';
  cAttributeCaption  = 'caption';
  cAttributeHint     = 'hint';
  cAttributeValue    = 'value';
  cAttributeText     = 'text';
  cAttributeName     = 'name';

Procedure GenerateXmlLngFile(filename:string);
var
  Xdoc: IXMLDocument;
  Node: IXMLNode;
  I: Integer;
begin
  if Filename='' then Filename:=cDefaultXML_LNGFileName;
  Xdoc:=TXMLDocument.Create(nil);
  try
    if FileExists(filename) then Xdoc.LoadFromFile(filename);
    if not Xdoc.Active then
    begin
      Xdoc.XML.Text:=cDefaultXMLhead;
      Xdoc.Active:=True;
    end;
    Node:=Xdoc.DocumentElement.ChildNodes.FindNode(cFormsElement);
    if Not Assigned(Node) then Node:=Xdoc.DocumentElement.AddChild(cFormsElement);
    for I:=0 to Application.ComponentCount-1 do
    begin
      if Application.Components[I] is TForm
       then SaveComponentBranch(Application.Components[I] as TControl,Node);
    end;
    Xdoc.SaveToFile(filename);
  finally
    Xdoc:=nil;
  end;
end;

Procedure SaveComponentBranch(Component:TControl; Node:IXMLNode);
{} Procedure SaveComboBox(Component:TComboBox; Node:IXMLNode);
{} var i:integer;
{} begin
{}   Node:=Node.AddChild(cItemsElement);
{}   For I:=0 to Component.Items.Count-1 do
{}     Node.AddChild(cItemElement).Attributes[cAttributeValue]:=Component.Items.Strings[i];
{} end;
var
  NodeX:IXMLNode;
  S:String;
  I:Integer;
begin
  if Not Assigned(Component) or not Assigned(Node) then exit;
    NodeX:=Node.AddChild((Component).Name);
    if Component is TComboBox then SaveComboBox((Component as TComboBox),NodeX);
    try
      try
        S:=GetPropValue(Component,'Caption');
        If S<>''
         then NodeX.Attributes[cAttributeCaption]:=S;
      except
      end;
      if Component.ShowHint
        then NodeX.Attributes[cAttributeHint]:=Component.Hint;
      if Component is TWinControl then
        For I:=0 to (Component as TWinControl).ControlCount-1
         do SaveComponentBranch((Component as TWinControl).Controls[i],NodeX);
    except
    end;
end;

{ TLanguage }

procedure TLanguage.BrowseComponent(Component: TControl; Node: IXMLNode);
{} function FindControl(Parent:TWinControl;Name:string):TControl;
{} var i:integer;
{} begin
{}   result:=nil;
{}   if (Parent=nil) then exit;
{}   for i:=0 to Parent.ControlCount-1 do
{}    if Parent.Controls[i].Name = Name then
{}    begin
{}      result:=Parent.Controls[i];
{}      break;
{}    end;
{} end;
var i:integer;
begin
  if Assigned(Component) and Assigned(Node) then
  begin
    if Component is TCustomCombo
     then LoadStringList(Component,Node);

    for i:=0 to Node.AttributeNodes.Count-1
     do SetPropValue(Component,Node.AttributeNodes[i].NodeName,Node.AttributeNodes[i].NodeValue);

    if Component is TWinControl then
    begin
      Node:=Node.ChildNodes.First;
      While Assigned(Node) do
      begin
        BrowseComponent(FindControl(Component as TWinControl,Node.NodeName),Node);
        Node:=Node.NextSibling;
      end;
    end;
  end;
end;

{ TLanguage }

constructor TLanguage.create;
begin
  inherited;
  FLanguages:=TStringList.Create;
end;

destructor TLanguage.Destroy;
begin
  FreeAndNil(FLanguages);
  inherited;
end;

function TLanguage.GetFileName: String;
begin
  if Assigned(XML)
   then Result:=XML.FileName;
end;

function TLanguage.getLanguages: TStringList;
begin
  Result:=FLanguages;
end;

procedure TLanguage.LoadLngFile(FileName: String);
var node:IXMLNode;
    S:String;
begin
  if (FileName<>'') and (FileName[1]='\') then
  begin
    Delete(FileName,1,1);
    FileName:=ExtractFilePath(Application.ExeName)+FileName;
  end;

  if not FileExists(FileName) then
  begin
    ShowMessage(Format(cLNGFileNotFount,[FileName]));
    Exit;
  end;

  XML:=TXMLDocument.Create(FileName);
  try
    if XML.Active then
    begin
      Node:=XML.DocumentElement.ChildNodes.First;
      While Assigned(Node) do
      begin
        if Node.NodeName=cFormsElement
         then LoadForms(Node);
        Node:=Node.NextSibling;
      end;
    end;
  finally
  end;
end;


procedure TLanguage.LoadForms(NodE:IXMLNode);
var
  NodeX:IXMLNode;
  I: Integer;
begin
  if not assigned(Node) then Exit;
  NodeX:=Node.ChildNodes.First;
  While Assigned(NodeX) do
  begin
    i:=0;
    while (i<Application.ComponentCount) and (Nodex.NodeName<>Application.Components[i].Name) do inc(i);
    if (i<Application.ComponentCount) and (Application.Components[i] is TControl)
     then BrowseComponent(Application.Components[i] as TControl,NodeX);
    NodeX:=NodeX.NextSibling;
  end;
end;

procedure TLanguage.LoadStringList(Component:TComponent; Node:IXMLNode);
begin
  Node:=Node.ChildNodes.FindNode(cItemsElement);
  If Assigned(Node) then
    begin
     (Component as TCustomCombo).Clear;
      Node:=Node.ChildNodes.First;
      While Assigned(Node) do
        begin
          If Node.NodeName=cItemElement then
            begin
              (Component as TCustomCombo).Items.Add(GetAttribute(Node,cAttributeValue,'<Error In Lng File>'));
            end;
          Node:=Node.NextSibling;
        end;
    end;
end;

procedure TLanguage.RefreshLanguages;
var i:integer;
begin
  FLanguages.Clear;
  for i:=0 to FLanguages.Count-1 do
    FLanguages.Objects[i].Free;
end;

procedure TLanguage.RewriteForm(Form: TForm);
Var Node:IXMLNode;
begin
  if assigned(XML) and XML.Active and Assigned(XML.DocumentElement) then
  begin
    Node:=XML.DocumentElement.ChildNodes.FindNode(cFormsElement);
    if Assigned(Node) then Node:=Node.ChildNodes.First;
    while Assigned(Node) and (UpperCase(Node.NodeName)<>UpperCase(Form.Name)) do Node:=Node.NextSibling;
    if Assigned(Node) then BrowseComponent(Form,Node);
  end;
end;

function TLanguage.text(key, defaultValue: string): String;
var Node:IXMLNode;
begin
  if Assigned(XML) and XML.Active then
  begin
    Node:=XML.DocumentElement.ChildNodes.First;
    while Assigned(Node) and (Node.NodeName<>cConstElement) do Node:=Node.NextSibling;

    if Assigned(Node)
     then Node:=Node.ChildNodes.FindNode(Key);
  end;

  Result:=GetAttribute(Node,cAttributeText,defaultValue);
end;

procedure TLanguage.TranslateForm(Form: TForm);
begin
  TranslateFormOrFrame(Form);
end;

procedure TLanguage.TranslateFormOrFrame(FormOrFrame: TScrollingWinControl);
var NodeX:IXMLNode;
begin
  if not assigned(FormOrFrame) or not Assigned(XML) or not Assigned(XML.DocumentElement) then Exit;
  NodeX:=XML.DocumentElement.ChildNodes.FindNode(cFormsElement);
  if Assigned(NodeX) then NodeX:=NodeX.ChildNodes.FindNode(FormOrFrame.Name);
  If Assigned(NodeX) then BrowseComponent(FormOrFrame,NodeX);
end;

procedure TLanguage.TranslateFrame(Frame: TFrame);
begin
  TranslateFormOrFrame(Frame);
end;

{ TLanguageItem }

constructor TLanguageItem.Create(FileName: String);
begin
  XML:=TXMLDocument.Create(FileName);
end;

function TLanguageItem.GetLngFileName: String;
begin
  Result:=XML.FileName;
end;

function TLanguageItem.GetLngName: String;
begin
  GetAttribute(XML.DocumentElement,cAttributeName,'?');
end;

initialization
  Language:=TLanguage.Create;
finalization
  Language.Free;
end.
